unit RarAdapter;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Dialogs, AbsArchiver;

type
  // Adapter for RAR
  TRarAdapter = class(TAbstractArchiver)
  private
    FArchiveName: string;
    FBaseDir: string;
    FSubFolders: Boolean;
    FSpecArgs: TStrings;
  protected
    function GetArchiveName: string; override;
    procedure SetArchiveName(const Value: string); override;
    function GetBaseDir: string; override;
    procedure SetBaseDir(const Value: string); override;
    function GetCompressLevel: TCompressLevel; override;
    procedure SetCompressLevel(const Value: TCompressLevel); override;
    function GetSpecArgs: TStrings; override;
    function GetSubFolders: Boolean; override;
    procedure SetSubFolders(const Value: Boolean); override;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Add; override;
    procedure Extract; override;
    procedure ExtractTo(const DirName: string); override;
  end;

implementation

{ TRarAdapter }

constructor TRarAdapter.Create;
begin
  FSpecArgs := TStringList.Create;
end;

destructor TRarAdapter.Destroy;
begin
  FSpecArgs.Free;
  inherited;
end;

// Is DOS  RAR.EXE Y
procedure TRarAdapter.Add;
var
  sSwitches: string;
  sFiles: string;
  sCmdLine: string;
  i: integer;
begin
  // إߩROCһݪѼ
  sSwitches := '-y ';       // Yes on all queries
  if FSubFolders then
    sSwitches := sSwitches + '-r ';    // Recurse subdirs
  if FBaseDir <> '' then
    sSwitches := sSwitches + '-w' + FBaseDir + ' '; // Work directory

  sFiles := ' ';
  for i := 0 to FSpecArgs.Count-1 do
    sFiles := sFiles + FSpecArgs[i] + ' ';

  sCmdLine := 'RAR a ' + sSwitches + ArchiveName + sFiles;
  WinExec(PChar(sCmdLine), SW_NORMAL);
end;

procedure TRarAdapter.Extract;
begin
  inherited;
  ShowMessage('TRarAdapter.Extract not implemented yet!');
end;

procedure TRarAdapter.ExtractTo(const DirName: string);
begin
  inherited;
  FBaseDir := DirName;
  Extract;
end;

function TRarAdapter.GetArchiveName: string;
begin
  Result := FArchiveName;
end;

function TRarAdapter.GetBaseDir: string;
begin
  Result := FBaseDir;
end;

function TRarAdapter.GetCompressLevel: TCompressLevel;
begin
  Result := clBest;
end;

function TRarAdapter.GetSpecArgs: TStrings;
begin
  Result := FSpecArgs;
end;

function TRarAdapter.GetSubFolders: Boolean;
begin
  Result := FSubFolders;
end;

procedure TRarAdapter.SetArchiveName(const Value: string);
begin
  FArchiveName := Value;
end;

procedure TRarAdapter.SetBaseDir(const Value: string);
begin
  FBaseDir := Value;
end;

procedure TRarAdapter.SetCompressLevel(const Value: TCompressLevel);
begin
end;

procedure TRarAdapter.SetSubFolders(const Value: Boolean);
begin
  FSubFolders := Value;
end;

initialization
  ArchiverFactory.RegisterClass(TRarAdapter);

end.
